@@ -36,6 +36,7 @@
 #include "load.h"
 #include "tree.h"
 #include "utils.h"
+#include "gb.h"
 
 typedef struct _GladeClipboardItem  GladeClipboardItem;
 
@@ -174,9 +175,10 @@ glade_clipboard_cut_or_copy (GladeClipboard *clipboard,
 			     GtkWidget      *widget,
 			     gboolean	     cut)
 {
-  GbWidgetGetArgData data;
+  GbWidgetGetArgData data = { 0 };
   gchar *old_locale, *saved_locale;
   GList *selection;
+  gboolean is_component;
 
   if (widget == NULL)
     {
@@ -205,13 +207,39 @@ glade_clipboard_cut_or_copy (GladeClipboard *clipboard,
   saved_locale = g_strdup (old_locale);
   setlocale (LC_NUMERIC, "C");
 
+  g_string_append (data.buffer,
+		   "<?xml version=\"1.0\" standalone=\"no\"?> <!--*- mode: xml -*-->\n"
+		   "<!DOCTYPE glade-interface SYSTEM \"http://glade.gnome.org/glade-project-2.0.dtd\">\n"
+		   "\n"
+		   "<glade-interface>\n");
+
+  /* If the widget is not a toplevel widget, we add a dummy parent to it.
+     This is so that we keep the packing properties of the widget. */
+  is_component = glade_util_is_component (widget);
+  if (!is_component)
+    {
+      g_string_append (data.buffer,
+		       "<widget class=\"GtkWindow\" id=\"glade-dummy-container\">\n");
+      data.indent++;
+    }
+
   gb_widget_save (widget, &data);
 
+  if (!is_component)
+    {
+      g_string_append (data.buffer,
+		       "</widget>\n");
+    }
+
+  g_string_append (data.buffer,
+		   "\n</glade-interface>\n");
+
   setlocale (LC_NUMERIC, saved_locale);
   g_free (saved_locale);
 
   if (data.error == NULL)
     {
+      g_print ("Adding to clipboard:\n%s\n", data.buffer->str);
       glade_clipboard_add (GLADE_CLIPBOARD (clipboard), project, cut,
 			   widget, data.buffer->str);
       if (cut)
@@ -225,16 +253,70 @@ glade_clipboard_cut_or_copy (GladeClipboard *clipboard,
 }
 
 
+static void
+initialize_all_widgets_cb (GtkWidget *widget, GbWidgetSetArgData *data)
+{
+  const gchar *widget_name;
+
+  /* If this is the widget we are replacing, just return. We don't want to
+     add it or its descendants to all_widgets. They will no longer exist, so
+     ATK relations can't use them. */
+  if (widget == data->replacing_widget)
+    return;
+
+  /* Add this widget to the all_widgets hash. */
+  if (GB_IS_GB_WIDGET (widget))
+    {
+      widget_name = gtk_widget_get_name (widget);
+      if (widget_name && *widget_name)
+	{
+#if 0
+	  g_print ("Initializing pointer to widget: %s, %p\n",
+		   widget_name, widget);
+#endif
+	  g_hash_table_insert (data->all_widgets, (gpointer) widget_name,
+			       widget);
+	}
+    }
+
+  /* Now recursively add the children. */
+  gb_widget_children_foreach (widget, (GtkCallback) initialize_all_widgets_cb,
+			      data);
+}
+
+
+/* This adds all widgets that are in the window being pasted into to the
+   all_widgets hash, so ATK relations to other widgets will be set when
+   pasting. */
+static void
+glade_clipboard_initialize_all_widgets (GbWidgetSetArgData *data)
+{
+  GtkWidget *toplevel;
+
+  /* If we aren't pasting into a component, just return. */
+  if (!data->replacing_widget)
+    return;
+
+  toplevel = glade_util_get_toplevel (data->replacing_widget);
+
+  initialize_all_widgets_cb (toplevel, data);
+}
+
+
+/* The widget argument is the widget to be pasted over, i.e. the widget in
+   the current interface that will be replaced by the clipboard item.
+   If it is NULL, the currently selected widget will be used. */
 void
 glade_clipboard_paste (GladeClipboard *clipboard,
 		       GladeProject   *project,
 		       GtkWidget      *widget)
 {
   GladeClipboardItem *item;
-  GbWidgetSetArgData data;
+  GbWidgetSetArgData data = { 0 };
   GtkWidget *parent, *new_widget = NULL;
   gchar *saved_locale, *saved_timezone, *child_name;
   GList *selection;
+  gboolean is_component = FALSE;
 
   item = glade_clipboard_get_current_item (clipboard);
   if (item == NULL)
@@ -247,12 +329,15 @@ glade_clipboard_paste (GladeClipboard *clipboard,
 	widget = GTK_WIDGET (selection->data);
     }
 
+  if (gtk_type_is_a (item->type, gtk_window_get_type ())
+      || gtk_type_is_a (item->type, gtk_menu_get_type ()))
+    is_component = TRUE;
+
   /* We can only paste toplevel components, i.e. windows and menus, into the
      project, and not into other widgets. */
   if (widget == NULL)
     {
-      if (!gtk_type_is_a (item->type, gtk_window_get_type ())
-	  && !gtk_type_is_a (item->type, gtk_menu_get_type ()))
+      if (!is_component)
 	{
 	  glade_util_show_message_box (_("You need to select a widget to paste into"), widget);
 	  return;
@@ -260,8 +345,7 @@ glade_clipboard_paste (GladeClipboard *clipboard,
     }
   else
     {
-      if (gtk_type_is_a (item->type, gtk_window_get_type ())
-	  || gtk_type_is_a (item->type, gtk_menu_get_type ()))
+      if (is_component)
 	widget = NULL;
     }
 
@@ -288,21 +372,19 @@ glade_clipboard_paste (GladeClipboard *clipboard,
       child_name = gb_widget_get_child_name (widget);
       if (child_name)
 	{
-	  if (!strcmp (child_name, "FileSel:ok_button")
-	      || !strcmp (child_name, "FileSel:cancel_button")
-	      || !strcmp (child_name, "ColorSel:ok_button")
-	      || !strcmp (child_name, "ColorSel:cancel_button")
-	      || !strcmp (child_name, "ColorSel:help_button")
-	      || !strcmp (child_name, "FontSel:ok_button")
-	      || !strcmp (child_name, "FontSel:cancel_button")
-	      || !strcmp (child_name, "FontSel:apply_button")
-	      || !strcmp (child_name, "InputDialog:save_button")
-	      || !strcmp (child_name, "InputDialog:close_button")
-
-	      || !strcmp (child_name, "Dialog:vbox")
-	      || !strcmp (child_name, "Dialog:action_area")
-	      || !strcmp (child_name, "GnomeDialog:vbox")
-	      || !strcmp (child_name, "GnomeDialog:action_area")
+	  if (!strcmp (child_name, GladeChildOKButton)
+	      || !strcmp (child_name, GladeChildCancelButton)
+	      || !strcmp (child_name, GladeChildHelpButton)
+	      || !strcmp (child_name, GladeChildApplyButton)
+	      || !strcmp (child_name, GladeChildSaveButton)
+	      || !strcmp (child_name, GladeChildCloseButton)
+	      || !strcmp (child_name, GladeChildDialogVBox)
+	      || !strcmp (child_name, GladeChildDialogActionArea)
+	      || !strcmp (child_name, GladeChildComboEntry)
+	      || !strcmp (child_name, GladeChildComboList)
+	      || !strcmp (child_name, GladeChildFontSelection)
+	      || !strcmp (child_name, GladeChildColorSelection)
+	      || !strcmp (child_name, GladeChildGnomeEntry)
 	      )
 	    {
 	      glade_util_show_message_box (_("You can't paste into the selected widget, since\nit is created automatically by its parent."), widget);
@@ -326,31 +408,31 @@ glade_clipboard_paste (GladeClipboard *clipboard,
 	  return;
 	}
 
-      /* Only buttons can be pasted into a GnomeDialog action area. */
+      /* Only buttons can be pasted into a dialog action area. */
       if (widget->parent)
 	{
 	  child_name = gb_widget_get_child_name (widget->parent);
-	  if (child_name && !strcmp (child_name, "GnomeDialog:action_area")
-	      && !gtk_type_is_a (item->type, gtk_button_get_type ()))
+	  if (child_name && !strcmp (child_name, GladeChildDialogActionArea)
+	      && item->type != GTK_TYPE_BUTTON)
 	    {
-	      glade_util_show_message_box (_("Only buttons can be pasted into a GnomeDialog action area."), widget);
+	      glade_util_show_message_box (_("Only buttons can be pasted into a dialog action area."), widget);
 	      return;
 	    }
 	}
 
 #ifdef USE_GNOME
-      if (GNOME_IS_DOCK (widget))
+      if (BONOBO_IS_DOCK (widget))
 	{
-	  if (!gtk_type_is_a (item->type, gnome_dock_item_get_type ()))
+	  if (!gtk_type_is_a (item->type, bonobo_dock_item_get_type ()))
 	    {
 	      glade_util_show_message_box (_("Only GnomeDockItem widgets can be pasted into a GnomeDock."), widget);
 	      return;
 	    }
 	}
 
-      if (GNOME_IS_DOCK_ITEM (widget))
+      if (BONOBO_IS_DOCK_ITEM (widget))
 	{
-	  if (!gtk_type_is_a (item->type, gnome_dock_item_get_type ()))
+	  if (!gtk_type_is_a (item->type, bonobo_dock_item_get_type ()))
 	    {
 	      glade_util_show_message_box (_("Only GnomeDockItem widgets can be pasted over a GnomeDockItem."), widget);
 	      return;
@@ -359,9 +441,9 @@ glade_clipboard_paste (GladeClipboard *clipboard,
 	  return;
 	}
 
-      if (gtk_type_is_a (item->type, gnome_dock_item_get_type ()))
+      if (gtk_type_is_a (item->type, bonobo_dock_item_get_type ()))
 	{
-	  if (!GNOME_IS_DOCK (widget) && !GNOME_IS_DOCK_ITEM (widget))
+	  if (!BONOBO_IS_DOCK (widget) && !BONOBO_IS_DOCK_ITEM (widget))
 	    {
 	      glade_util_show_message_box (_("GnomeDockItem widgets can only be pasted into a GnomeDock."), widget);
 	      return;
@@ -371,11 +453,10 @@ glade_clipboard_paste (GladeClipboard *clipboard,
     }
 
   data.project = project;
-  data.fp = NULL;
   data.filename = NULL;
   data.xml_buffer = item->xml_data;
-  data.xml_buffer_pos = 0;
-  data.xml_buffer_len = strlen (data.xml_buffer);
+  data.status = GLADE_STATUS_OK;
+  data.all_widgets = g_hash_table_new (g_str_hash, g_str_equal);
 
   /* We normally create new names for the widgets pasted, discarding the names
      in the XML data. However, if the widgets were cut to the clipboard, and
@@ -405,9 +486,8 @@ glade_clipboard_paste (GladeClipboard *clipboard,
     {
       if (GTK_IS_FIXED (widget)
 	  || GTK_IS_LAYOUT (widget)
-	  || GTK_IS_PACKER (widget)
 #ifdef USE_GNOME
-	  || GNOME_IS_DOCK (widget)
+	  || BONOBO_IS_DOCK (widget)
 #endif
 	  )
 	{
@@ -420,7 +500,11 @@ glade_clipboard_paste (GladeClipboard *clipboard,
 	}
     }
 
-  load_init_before_read (GLADE_PASTE_BUFFER_INCREMENT, &data);
+  /* Initialize the all_widgets hash with all widget names in the component,
+     except the widget being replaced and any of its descendants. This is
+     done so that ATK relations to targets in the component will still be OK.
+  */
+  glade_clipboard_initialize_all_widgets (&data);
 
   tree_freeze ();
 
@@ -432,15 +516,44 @@ glade_clipboard_paste (GladeClipboard *clipboard,
   saved_timezone = glade_util_set_timezone ("UTC");
 
   /* Now parse the clipboard data. */
-  load_token_skip_whitespace (&data);
-  if (data.status == GLADE_STATUS_OK)
+  data.interface = glade_parser_parse_buffer (data.xml_buffer,
+					      strlen (data.xml_buffer), NULL);
+  if (!data.interface || data.interface->n_toplevels != 1)
     {
-      if (!strcmp (data.buffer.ptr + data.token, "widget"))
-	new_widget = gb_widget_load (NULL, &data, parent);
-      else
-	data.status = GLADE_STATUS_INVALID_ENTITY;
+      /* I don't think this should happen, except due to bugs. */
+      g_warning ("Error pasting from clipboard");
+      glade_interface_destroy (data.interface);
+      g_hash_table_destroy (data.all_widgets);
+      return;
     }
 
+  /* If the widget is a toplevel component, we use the top widget info and
+     no child info. If not, we use the first child, as the toplevel will be
+     a dummy container. */
+  data.child_info = NULL;
+  data.widget_info = data.interface->toplevels[0];
+
+  if (!is_component)
+    {
+      if (data.widget_info->n_children != 1
+	  || data.widget_info->children == NULL)
+	{
+	  data.status = GLADE_STATUS_ERROR;
+	}
+      else
+	{
+	  data.child_info = &data.widget_info->children[0];
+	  data.widget_info = data.child_info->child;
+	}
+    }
+
+  /* Create the widget. */
+  if (data.status == GLADE_STATUS_OK)
+    new_widget = gb_widget_load (NULL, &data, parent);
+
+  /* Destroy the parse data. */
+  glade_interface_destroy (data.interface);
+
   /* Reset the timezone. */
   glade_util_reset_timezone (saved_timezone);
 
@@ -450,9 +563,7 @@ glade_clipboard_paste (GladeClipboard *clipboard,
 
   tree_thaw ();
 
-  g_free (data.buffer.ptr);
-  gb_free_load_properties (&data.properties);
-  gb_free_load_properties (&data.child_properties);
+  g_hash_table_destroy (data.all_widgets);
 
   if (data.status != GLADE_STATUS_OK)
     {
@@ -484,7 +595,7 @@ glade_clipboard_add	(GladeClipboard *clipboard,
   GbWidget *gbwidget;
   gchar *name;
 
-  name = gtk_widget_get_name (widget);
+  name = (char*) gtk_widget_get_name (widget);
   gbwidget = gb_widget_lookup (widget);
   g_return_if_fail (gbwidget != NULL);
 
@@ -497,7 +608,7 @@ glade_clipboard_add	(GladeClipboard *clipboard,
   /* Connect to the project's destroy signal to set the pointer to NULL, so
      we never have invalid pointers. */
   gtk_signal_connect (GTK_OBJECT (project), "destroy",
-		      glade_clipboard_on_project_destroy,
+		      GTK_SIGNAL_FUNC (glade_clipboard_on_project_destroy),
 		      item);
 
   gtk_clist_insert (GTK_CLIST (clipboard->clist), 0, &name);
