@@ -32,6 +32,7 @@
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkradiobutton.h>
+#include <gtk/gtkstock.h>
 #include <gtk/gtkvbox.h>
 
 #include "gladeconfig.h"
@@ -360,14 +361,17 @@ glade_project_options_init (GladeProjectOptions * options)
   menu = gtk_menu_new ();
   menuitem = gtk_menu_item_new_with_label ("GNU General Public License");
   gtk_widget_show (menuitem);
-  gtk_menu_append (GTK_MENU (menu), menuitem);
+  gtk_container_add (GTK_CONTAINER (menu), menuitem);
   gtk_option_menu_set_menu (GTK_OPTION_MENU (options->license_option_menu),
 			    menu);
 #endif
 
   /* Source Language. */
   frame = gtk_frame_new (_("Language:"));
+  /* Hide this for now, until code generators have been ported to GNOME 2. */
+#if 0
   gtk_widget_show (frame);
+#endif
   gtk_box_pack_start (GTK_BOX (general_options_vbox), frame, FALSE, TRUE, 4);
 
   alignment = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
@@ -403,6 +407,8 @@ glade_project_options_init (GladeProjectOptions * options)
   gtk_container_add (GTK_CONTAINER (frame), hbox);
 
   options->gnome_support = gtk_check_button_new_with_label (_("Enable Gnome Support"));
+  /* This can't be changed now. */
+  gtk_widget_set_sensitive (options->gnome_support, FALSE);
   gtk_widget_show (options->gnome_support);
   gtk_box_pack_start (GTK_BOX (hbox), options->gnome_support, TRUE, TRUE, 0);
   gtk_tooltips_set_tip (tooltips, options->gnome_support,
@@ -714,11 +720,15 @@ glade_project_options_init (GladeProjectOptions * options)
   gtk_button_box_set_layout (GTK_BUTTON_BOX (hbbox), GTK_BUTTONBOX_END);
   gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbbox), 8);
 
-#ifdef USE_GNOME
-  options->ok_button = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
-#else
-  options->ok_button = gtk_button_new_with_label (_("OK"));
-#endif
+  options->cancel_button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+  GTK_WIDGET_SET_FLAGS (options->cancel_button, GTK_CAN_DEFAULT);
+  gtk_widget_show (options->cancel_button);
+  gtk_container_add (GTK_CONTAINER (hbbox), options->cancel_button);
+  gtk_signal_connect_object (GTK_OBJECT (options->cancel_button), "clicked",
+			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
+			     GTK_OBJECT (options));
+
+  options->ok_button = gtk_button_new_from_stock (GTK_STOCK_OK);
   GTK_WIDGET_SET_FLAGS (options->ok_button, GTK_CAN_DEFAULT);
   gtk_widget_show (options->ok_button);
   gtk_container_add (GTK_CONTAINER (hbbox), options->ok_button);
@@ -726,20 +736,10 @@ glade_project_options_init (GladeProjectOptions * options)
   gtk_signal_connect (GTK_OBJECT (options->ok_button), "clicked",
 		      GTK_SIGNAL_FUNC (glade_project_options_ok), options);
 
-#ifdef USE_GNOME
-  options->cancel_button = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
-#else
-  options->cancel_button = gtk_button_new_with_label (_("Cancel"));
-#endif
-  GTK_WIDGET_SET_FLAGS (options->cancel_button, GTK_CAN_DEFAULT);
-  gtk_widget_show (options->cancel_button);
-  gtk_container_add (GTK_CONTAINER (hbbox), options->cancel_button);
-  gtk_signal_connect_object (GTK_OBJECT (options->cancel_button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (options));
   gtk_signal_connect (GTK_OBJECT (options), "key_press_event",
 		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
 		      GINT_TO_POINTER (GladeEscDestroys));
+
   gtk_widget_grab_focus (options->directory_entry);
 }
 
@@ -766,12 +766,16 @@ glade_project_options_destroy (GtkObject *object)
   options = GLADE_PROJECT_OPTIONS (object);
 
   g_free (options->language_buttons);
+  options->language_buttons = NULL;
 
   /* FIXME: Maybe we need to disconnect the destroy signal handler added to
      the project. */
 
   if (options->filesel)
-    gtk_widget_destroy (options->filesel);
+    {
+      gtk_widget_destroy (options->filesel);
+      options->filesel = NULL;
+    }
 }
 
 
@@ -1041,7 +1045,7 @@ glade_project_options_generate_name (GladeProjectOptions *options)
 {
   gchar *directory, *project_name;
 
-  directory = gtk_entry_get_text (GTK_ENTRY (options->directory_entry));
+  directory = (char*) gtk_entry_get_text (GTK_ENTRY (options->directory_entry));
   project_name = glade_project_options_generate_project_name_from_directory (directory);
   options->auto_generation_level++;
   gtk_entry_set_text (GTK_ENTRY (options->name_entry), project_name);
@@ -1070,7 +1074,7 @@ glade_project_options_generate_program_name (GladeProjectOptions *options)
 {
   gchar *project_name, *program_name;
 
-  project_name = gtk_entry_get_text (GTK_ENTRY (options->name_entry));
+  (char*) project_name = gtk_entry_get_text (GTK_ENTRY (options->name_entry));
   program_name = glade_project_options_generate_program_name_from_project_name (project_name);
   options->auto_generation_level++;
   gtk_entry_set_text (GTK_ENTRY (options->program_name_entry), program_name);
@@ -1106,7 +1110,7 @@ glade_project_options_generate_xml_filename (GladeProjectOptions *options)
 {
   gchar *program_name, *xml_filename;
 
-  program_name = gtk_entry_get_text (GTK_ENTRY (options->program_name_entry));
+  program_name = (char*) gtk_entry_get_text (GTK_ENTRY (options->program_name_entry));
   xml_filename = glade_project_options_generate_xml_filename_from_program_name (program_name);
   options->auto_generation_level++;
   gtk_entry_set_text (GTK_ENTRY (options->xml_filename_entry), xml_filename);
@@ -1173,7 +1177,7 @@ glade_project_options_show_file_selection (GtkWidget *widget,
 		      GINT_TO_POINTER (GladeEscCloses));
     }
 
-  filename = gtk_entry_get_text (GTK_ENTRY (options->directory_entry));
+  filename = (char*) gtk_entry_get_text (GTK_ENTRY (options->directory_entry));
   if (glade_util_file_exists (filename))
     {
       gtk_file_selection_set_filename (GTK_FILE_SELECTION (options->filesel),
@@ -1194,7 +1198,7 @@ glade_project_options_set_filename (GtkWidget *widget,
 {
   gchar *filename;
 
-  filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (options->filesel));
+  filename = (char*) gtk_file_selection_get_filename (GTK_FILE_SELECTION (options->filesel));
   gtk_entry_set_text (GTK_ENTRY (options->directory_entry), filename);
   glade_util_close_window (options->filesel);
 }
@@ -1304,7 +1308,7 @@ get_entry_text (GtkWidget *entry)
 {
   gchar *text;
 
-  text = gtk_entry_get_text (GTK_ENTRY (entry));
+  text = (char*) gtk_entry_get_text (GTK_ENTRY (entry));
   if (text && text[0] != '\0')
     return text;
   else
