@@ -16,6 +16,8 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "gladeconfig.h"
+
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -31,12 +33,12 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include "gladeconfig.h"
-
 #ifdef USE_GNOME
 #include <gnome.h>
+#include <bonobo.h>
 #endif
 
+#include "gb.h"
 #include "gbwidget.h"
 #include "glade_project_window.h"
 #include "utils.h"
@@ -46,17 +48,14 @@ gchar *GladeDayNames[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
 gchar *GladeMonthNames[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
 			     "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
 
+typedef void (*GladeDialogCallback) (GtkDialog *dialog, gpointer data);
+
 #define	READ_BUFFER_SIZE	4096
 
 static void on_entry_dialog_ok (GtkWidget * widget,
 				gpointer data);
 static void on_entry_dialog_destroy (GtkWidget * widget,
 				     gpointer data);
-#ifdef USE_GNOME
-static void gnome_dialog_button_clicked (GnomeDialog *dialog,
-					 gint button,
-					 gpointer data);
-#endif
 static gboolean check_components_match (const gchar *base,
 					const gchar *path,
 					gint         root_pos,
@@ -104,135 +103,20 @@ void
 glade_util_show_message_box	(const gchar	*message,
 				 GtkWidget	*transient_widget)
 {
-  GtkWidget *toplevel;
+  GtkWidget *dialog;
+  GtkWindow *transient_parent;
 
-#ifdef USE_GNOME
-  GtkWidget *dialog, *button;
-  dialog = gnome_message_box_new (message, GNOME_MESSAGE_BOX_ERROR,
-				  GNOME_STOCK_BUTTON_OK, NULL);
-  button = GNOME_DIALOG (dialog)->buttons->data;
-  gtk_widget_grab_focus (button);
-#else
-  GtkWidget *dialog, *label, *button;
-
-  dialog = gtk_dialog_new ();
+  transient_parent = (GtkWindow *)glade_util_get_toplevel (transient_widget);
+  dialog = gtk_message_dialog_new (transient_parent,
+				   GTK_DIALOG_MODAL,
+				   GTK_MESSAGE_ERROR,
+				   GTK_BUTTONS_OK,
+				   "%s", message);
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
-  gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 
-  label = gtk_label_new (message);
-  gtk_misc_set_padding (GTK_MISC (label), 20, 20);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label,
-		      TRUE, TRUE, 0);
-  gtk_widget_show (label);
-
-  button = gtk_button_new_with_label (_("OK"));
-  gtk_widget_set_usize (button, 80, -1);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
-		      FALSE, FALSE, 14);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_grab_default (button);
-  gtk_widget_grab_focus (button);
-  gtk_widget_show (button);
-
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (dialog));
-  gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
-		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
-		      GINT_TO_POINTER (GladeEscDestroys));
-#endif
-
-  if (transient_widget)
-    {
-      toplevel = glade_util_get_toplevel (transient_widget);
-      if (toplevel && GTK_IS_WINDOW (toplevel) && GTK_WIDGET_MAPPED (toplevel))
-	gtk_window_set_transient_for (GTK_WINDOW (dialog),
-				      GTK_WINDOW (toplevel));
-    }
-
-  gtk_widget_show (dialog);
-}
-
-
-/* This shows a dialog box with a message and a number of buttons.
- * Signal handlers can be supplied for any of the buttons.
- * NOTE: The dialog is automatically destroyed when any button is clicked.
- * default_button specifies the default button, numbered from 1..
- * data is passed to the signal handler.
-
-   Example usage:
-     GtkWidget *dialog;
-     gchar *buttons[] = { "Yes", "No", "Cancel" };
-     GtkSignalFunc signal_handlers[] = { on_yes, on_no, NULL };
-
-     dialog = glade_util_create_dialog_with_buttons ("Do you want to save the current project?", 3, buttons, 3, signal_handlers, NULL, NULL);
-     gtk_widget_show (dialog);
- */
-GtkWidget*
-glade_util_create_dialog_with_buttons	(const gchar	*message,
-					 gint		 nbuttons,
-					 const gchar	*buttons[],
-					 gint		 default_button,
-					 GtkSignalFunc	 signal_handlers[],
-					 gpointer	 data,
-					 GtkWidget	*transient_widget)
-{
-  GtkWidget *dialog, *hbox, *label, *button, *bbox, *toplevel;
-  int i;
-
-  dialog = gtk_dialog_new ();
-  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
-  gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-
-  hbox = gtk_hbox_new (FALSE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (hbox), 20);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
-		      TRUE, TRUE, 0);
-  gtk_widget_show (hbox);
-
-  label = gtk_label_new (message);
-  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
-  gtk_widget_show (label);
-
-  bbox = gtk_hbutton_box_new ();
-  gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
-  gtk_button_box_set_spacing (GTK_BUTTON_BOX (bbox), 10);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), bbox,
-		      FALSE, TRUE, 0);
-  gtk_widget_show (bbox);
-
-  for (i = 0; i < nbuttons; i++)
-    {
-      button = gtk_button_new_with_label (_(buttons[i]));
-      gtk_container_add (GTK_CONTAINER (bbox), button);
-      GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-      if (i == default_button - 1) {
-	gtk_widget_grab_default (button);
-	gtk_widget_grab_focus (button);
-      }
-      gtk_widget_show (button);
-
-      if (signal_handlers[i])
-	gtk_signal_connect (GTK_OBJECT (button), "clicked", signal_handlers[i],
-			    data);
-
-      gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-				 GTK_SIGNAL_FUNC (gtk_widget_destroy),
-				 GTK_OBJECT (dialog));
-      gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
-			  GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
-			  GINT_TO_POINTER (GladeEscDestroys));
-    }
-
-  if (transient_widget)
-    {
-      toplevel = glade_util_get_toplevel (transient_widget);
-      if (toplevel && GTK_IS_WINDOW (toplevel) && GTK_WIDGET_MAPPED (toplevel))
-	gtk_window_set_transient_for (GTK_WINDOW (dialog),
-				      GTK_WINDOW (toplevel));
-    }
-
-  return dialog;
+  gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
 }
 
 
@@ -288,7 +172,16 @@ glade_util_show_entry_dialog	(const gchar	*message,
   gtk_widget_grab_focus (entry);
   gtk_object_set_data (GTK_OBJECT (dialog), "entry", entry);
 
-  button = gtk_button_new_with_label (_("OK"));
+  button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
+		      FALSE, TRUE, 20);
+  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+  gtk_widget_show (button);
+  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
+			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
+			     GTK_OBJECT (dialog));
+
+  button = gtk_button_new_from_stock (GTK_STOCK_OK);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
 		      FALSE, TRUE, 20);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
@@ -298,14 +191,6 @@ glade_util_show_entry_dialog	(const gchar	*message,
 		      GTK_SIGNAL_FUNC (on_entry_dialog_ok),
 		      data);
 
-  button = gtk_button_new_with_label (_("Cancel"));
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
-		      FALSE, TRUE, 20);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_show (button);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (dialog));
   gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
 		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
 		      GINT_TO_POINTER (GladeEscDestroys));
@@ -346,7 +231,7 @@ on_entry_dialog_ok (GtkWidget * widget, gpointer data)
   widget = GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (dialog), "widget"));
   entry = GTK_WIDGET (gtk_object_get_data (GTK_OBJECT (dialog), "entry"));
   g_return_if_fail (entry != NULL);
-  text = gtk_entry_get_text (GTK_ENTRY (entry));
+  text = (gchar*) gtk_entry_get_text (GTK_ENTRY (entry));
 
   if (handler)
     close = (*handler) (widget, text, data);
@@ -354,6 +239,21 @@ on_entry_dialog_ok (GtkWidget * widget, gpointer data)
     gtk_widget_destroy (dialog);
 }
 
+static void
+on_dialog_response (GtkDialog *dialog, gint response, gpointer ok_handler_data)
+{
+  GladeDialogCallback ok_handler;
+
+  if (response == GTK_RESPONSE_OK)
+    {
+      ok_handler = (GladeDialogCallback) g_object_get_data (G_OBJECT (dialog), "ok_handler");
+      ok_handler (dialog, ok_handler_data);
+    }
+  else
+    {
+      gtk_widget_destroy (GTK_WIDGET (dialog));
+    }
+}
 
 /* This creates a dialog with OK & Cancel buttons, usable in plain GTK or
    Gnome, and returns a vbox which the caller can place widgets in.
@@ -368,57 +268,24 @@ glade_util_create_dialog	(const gchar	   *title,
 				 gpointer           ok_handler_data,
 				 GtkWidget	  **vbox)
 {
-#ifdef USE_GNOME
   GtkWidget *dialog;
+  GtkWindow *transient_parent;
 
-  dialog = gnome_dialog_new (title, GNOME_STOCK_BUTTON_OK,
-			     GNOME_STOCK_BUTTON_CANCEL, NULL);
-  gtk_object_set_data (GTK_OBJECT (dialog), "ok_handler", ok_handler);
-  gtk_signal_connect (GTK_OBJECT (dialog), "clicked",
-		      gnome_dialog_button_clicked, ok_handler_data);
-  gnome_dialog_set_default (GNOME_DIALOG (dialog), 0);
+  transient_parent = (GtkWindow *)glade_util_get_toplevel (transient_for);
+  dialog = gtk_dialog_new_with_buttons (title, transient_parent,
+					0,
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					GTK_STOCK_OK, GTK_RESPONSE_OK,
+					NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
-  *vbox = GNOME_DIALOG (dialog)->vbox;
-#else
-  GtkWidget *dialog, *buttonbox, *button;
+  g_object_set_data (G_OBJECT (dialog), "ok_handler", ok_handler);
 
-  dialog = gtk_dialog_new ();
-  gtk_window_set_title (GTK_WINDOW (dialog), title);
-  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
+  g_signal_connect (dialog, "response",
+		    G_CALLBACK (on_dialog_response),
+		    ok_handler_data);
 
-  buttonbox = gtk_hbutton_box_new ();
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area),
-		     buttonbox);
-  gtk_widget_show (buttonbox);
-
-  button = gtk_button_new_with_label (_("OK"));
-  gtk_container_add (GTK_CONTAINER (buttonbox), button);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_grab_default (button);
-  gtk_widget_show (button);
-  gtk_signal_connect (GTK_OBJECT (button), "clicked", ok_handler,
-		      ok_handler_data);
-
-  button = gtk_button_new_with_label (_("Cancel"));
-  gtk_container_add (GTK_CONTAINER (buttonbox), button);
-  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-  gtk_widget_show (button);
-  gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (dialog));
-  gtk_signal_connect (GTK_OBJECT (dialog), "key_press_event",
-		      GTK_SIGNAL_FUNC (glade_util_check_key_is_esc),
-		      GINT_TO_POINTER (GladeEscDestroys));
   *vbox = GTK_DIALOG (dialog)->vbox;
-#endif
-
-  if (transient_for)
-    {
-      GtkWidget *transient_parent = glade_util_get_toplevel (transient_for);
-      if (GTK_IS_WINDOW (transient_parent))
-	gtk_window_set_transient_for (GTK_WINDOW (dialog),
-				      GTK_WINDOW (transient_parent));
-    }
 
   return dialog;
 }
@@ -451,26 +318,6 @@ gint glade_util_check_key_is_esc (GtkWidget *widget,
     return FALSE;
 }
 
-
-#ifdef USE_GNOME
-static void gnome_dialog_button_clicked (GnomeDialog *dialog,
-					 gint button,
-					 gpointer data)
-{
-  GtkSignalFunc ok_handler;
-
-  if (button == 0)
-    {
-      ok_handler = (GtkSignalFunc) gtk_object_get_data (GTK_OBJECT (dialog), "ok_handler");
-      (*ok_handler) (dialog, data);
-    }
-  else
-    {
-      gtk_widget_destroy (GTK_WIDGET (dialog));
-    }
-}
-#endif /* USE_GNOME */
-
 /* This returns a new entry ready to insert in a dialog.
    The entry is set up so that <Return> will invoke the default action. The
    returned widget must be added to a container in the dialog. */ 
@@ -736,6 +583,9 @@ glade_util_get_toplevel (GtkWidget *widget)
 {
   GtkWidget *parent;
 
+  if (widget == NULL)
+    return NULL;
+
   for (;;)
     {
       if (GTK_IS_MENU (widget))
@@ -760,13 +610,21 @@ glade_util_get_parent	(GtkWidget	   *widget)
   for (;;)
     {
       if (GTK_IS_MENU (widget))
-        parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
+	parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
       else
-        parent = widget->parent;
+	parent = widget->parent;
+
+      /* There is no way to move up from the combo popup list window to the
+	 combo, so we added a special pointer. */
+      if (!parent)
+	parent = gtk_object_get_data (GTK_OBJECT (widget), GladeParentKey);
+
       if (parent == NULL)
         break;
+
       if (GB_IS_GB_WIDGET (parent))
 	return parent;
+
       widget = parent;
     }
 
@@ -775,30 +633,14 @@ glade_util_get_parent	(GtkWidget	   *widget)
 
 
 /* This is used when setting up keyboard accelerators resulting from underlined
-   keys in labels. It returns the widget to add the accelerator to, and the
-   signal to emit. If the label is in a button, then the accelerator is
-   connected to the button's "activate" signal. If not, it tries to find the
-   widget to the right of the label, and connects to its "set_focus" signal. */
+   keys in labels. It returns the default mnemonic widget to use. */
 GtkWidget*
-glade_util_find_default_accelerator_target	(GtkWidget	   *label,
-						 gchar		  **signal)
+glade_util_find_default_accelerator_target	(GtkWidget	   *label)
 {
   GtkWidget *parent, *child, *target = NULL;
 
   g_return_val_if_fail (GTK_IS_LABEL (label), NULL);
 
-  /* First see if any ancestor of the label is a button. */
-  parent = label->parent;
-  while (parent)
-    {
-      if (GTK_IS_BUTTON (parent))
-	{
-	  *signal = "clicked";
-	  return parent;
-	}
-      parent = parent->parent;
-    }
-
   /* Now try to find the widget to the right of the label. We can do this
      fairly easily for tables, boxes, and fixed containers. But the packer
      container may be more difficult. */
@@ -819,7 +661,6 @@ glade_util_find_default_accelerator_target	(GtkWidget	   *label,
 
       if (target)
 	{
-	  *signal = "grab_focus";
 	  return target;
 	}
 
@@ -865,21 +706,6 @@ glade_util_find_widget_recursive (GtkWidget *widget,
 }
 
 
-/* This looks up the hierarchy, starting from widget, trying to find a
-   GtkButton or subclass. It returns the button widget or NULL if not found. */
-GtkWidget*
-glade_util_find_parent_button (GtkWidget *widget)
-{
-  while (widget)
-    {
-      if (GTK_IS_BUTTON (widget))
-	return widget;
-      widget = widget->parent;
-    }
-  return NULL;
-}
-
-
 /* This tries to find the widget on the right of the given child. */
 static GtkWidget*
 glade_util_find_table_accelerator_target (GtkWidget *parent, GtkWidget *child)
@@ -1075,8 +901,6 @@ glade_util_find_focus_child (GtkWidget *widget)
 #ifdef USE_GNOME
   if (GNOME_IS_FILE_ENTRY (widget))
     return gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (widget));
-  if (GNOME_IS_NUMBER_ENTRY (widget))
-    return gnome_number_entry_gtk_entry (GNOME_NUMBER_ENTRY (widget));
 #endif
 
   return NULL;
@@ -1165,6 +989,7 @@ glade_util_find_fixed_child	(GtkFixed	   *fixed,
 }
 
 
+#if GLADE_SUPPORTS_GTK_PACKER
 GtkPackerChild *
 glade_util_find_packer_child (GtkPacker * packer,
 			      GtkWidget * widget)
@@ -1183,6 +1008,7 @@ glade_util_find_packer_child (GtkPacker * packer,
   g_warning (_("Widget not found in packer"));
   return NULL;
 }
+#endif
 
 
 /* This finds the toolbar structure representing the given child, or NULL
@@ -1215,11 +1041,24 @@ glade_util_find_toolbar_child (GtkWidget *toolbar,
 
 
 /* This returns the GtkLabel's text, including the underline characters.
-   It is needed since GtkLabel doesn't provide the opposite function to
-   gtk_label_parse_uline(). The returned string should be freed after use. */
+   We don't really need it now for GTK+ 2.0. Free the returned string. */
 gchar*
 glade_util_get_label_text (GtkWidget *label)
 {
+  char *result;
+
+  /* Hopefully this will work in GTK+ 2.0. */
+  result = (gchar*) gtk_label_get_label (GTK_LABEL (label));
+
+#if 0
+  g_print ("Label text: %s\n", result);
+#endif
+
+  /* We strdup it just to remain compatable with the old version. */
+  return g_strdup (result);
+
+#if 0
+  /* This is the old GTK+ 1.2 version, for reference. */
   GdkWChar *label_wc, *label_with_underscores;
   gchar *label_text, *label_pattern, *result;
   gint len, i, j;
@@ -1259,6 +1098,7 @@ glade_util_get_label_text (GtkWidget *label)
   g_free (label_with_underscores);
 
   return result;
+#endif
 }
 
 
@@ -1765,7 +1605,7 @@ glade_util_parent_directory	(const gchar	    *dir)
   g_return_val_if_fail (g_path_is_absolute (dir), NULL);
 
   /* We handle the root dir specially here, and return NULL. */
-  skipped_root = g_path_skip_root ((gchar*) dir);
+  skipped_root = (gchar*) g_path_skip_root ((gchar*) dir);
   if (*skipped_root == '\0')
     return NULL;
 
@@ -1932,3 +1772,362 @@ glade_util_reset_timezone	(gchar		*tz)
 }
 
 
+gboolean
+glade_util_check_is_stock_id	(const gchar *stock_id)
+{
+  static GHashTable *stock_hash = NULL;
+
+  gboolean retval = FALSE;
+
+  /* Create a hash of stock ids, so future calls will be fast. */
+  if (!stock_hash)
+    {
+      GSList *stock_items, *elem;
+
+      stock_hash = g_hash_table_new (g_str_hash, g_str_equal);
+
+      stock_items = gtk_stock_list_ids ();
+      for (elem = stock_items; elem; elem = elem->next)
+	{
+	  /* We just use the stock id for the key and the data, as we don't
+	     need to store anything. We should free the keys at some point,
+	     but we need them until Glade exits. */
+	  g_hash_table_insert (stock_hash, elem->data, elem->data);
+	}
+
+      g_slist_free (stock_items);
+    }
+
+  if (stock_id && g_hash_table_lookup (stock_hash, stock_id))
+    retval = TRUE;
+
+#if 0
+  g_print ("glade_util_check_is_stock_id: %s -> %i\n",
+	   stock_id ? stock_id : "", retval);
+#endif
+
+  return retval;
+}
+
+
+static gint
+compare_uline_labels (const gchar *labela, const gchar *labelb)
+{
+  for (;;) {
+    gunichar c1, c2;
+
+    if (*labela == '\0')
+      return (*labelb == '\0') ? 0 : -1;
+    if (*labelb == '\0')
+      return 1;
+
+    c1 = g_utf8_get_char (labela);
+    if (c1 == '_')
+      {
+	labela = g_utf8_next_char (labela);
+	c1 = g_utf8_get_char (labela);
+      }
+
+    c2 = g_utf8_get_char (labelb);
+    if (c2 == '_')
+      {
+	labelb = g_utf8_next_char (labelb);
+	c2 = g_utf8_get_char (labelb);
+      }
+
+    if (c1 < c2)
+      return -1;
+    if (c1 > c2)
+      return 1;
+
+    labela = g_utf8_next_char (labela);
+    labelb = g_utf8_next_char (labelb);
+  }
+
+  /* Shouldn't be reached. */
+  return 0;
+}
+
+
+/* This is a GCompareFunc for comparing the labels of 2 stock items, ignoring
+   any '_' characters. It isn't particularly efficient. */
+gint
+glade_util_compare_stock_labels (gconstpointer a, gconstpointer b)
+{
+  const gchar *stock_ida = a, *stock_idb = b;
+  GtkStockItem itema, itemb;
+  gboolean founda, foundb;
+  gint retval;
+
+  founda = gtk_stock_lookup (stock_ida, &itema);
+  foundb = gtk_stock_lookup (stock_idb, &itemb);
+
+  if (founda)
+    {
+      if (!foundb)
+	retval = -1;
+      else
+	/* FIXME: Not ideal for UTF-8. */
+	retval = compare_uline_labels (itema.label, itemb.label);
+    }
+  else
+    {
+      if (!foundb)
+	retval = 0;
+      else
+	retval = 1;
+    }
+
+  return retval;
+}
+
+
+/* These are pinched from gtkcombo.c */
+gpointer /* GtkListItem *  */
+glade_util_gtk_combo_find (GtkCombo * combo)
+{
+  gchar *text;
+  gchar *ltext;
+  GList *clist;
+  int (*string_compare) (const char *, const char *);
+
+  if (combo->case_sensitive)
+    string_compare = strcmp;
+  else
+    string_compare = g_strcasecmp;
+
+  text = (gchar*) gtk_entry_get_text (GTK_ENTRY (combo->entry));
+  clist = GTK_LIST (combo->list)->children;
+
+  while (clist && clist->data)
+    {
+      ltext = glade_util_gtk_combo_func (GTK_LIST_ITEM (clist->data));
+      if (!ltext)
+	continue;
+      if (!(*string_compare) (ltext, text))
+	return (GtkListItem *) clist->data;
+      clist = clist->next;
+    }
+
+  return NULL;
+}
+
+gchar *
+glade_util_gtk_combo_func (gpointer data)
+{
+  GtkListItem * listitem = data;
+
+  /* I needed to pinch this as well - Damon. */
+  static const gchar *gtk_combo_string_key = "gtk-combo-string-value";
+
+  GtkWidget *label;
+  gchar *ltext = NULL;
+
+  ltext = (gchar *) gtk_object_get_data (GTK_OBJECT (listitem),
+					 gtk_combo_string_key);
+  if (!ltext)
+    {
+      label = GTK_BIN (listitem)->child;
+      if (!label || !GTK_IS_LABEL (label))
+	return NULL;
+      gtk_label_get (GTK_LABEL (label), &ltext);
+    }
+  return ltext;
+}
+
+/**
+ * glade_enum_from_string
+ * @type: the GType for this enum type.
+ * @string: the string representation of the enum value.
+ *
+ * This helper routine is designed to be used by widget build routines to
+ * convert the string representations of enumeration values found in the
+ * XML descriptions to the integer values that can be used to configure
+ * the widget.
+ *
+ * Returns: the integer value for this enumeration, or 0 if it couldn't be
+ * found.
+ */
+gint
+glade_enum_from_string (GType type, const char *string)
+{
+    GEnumClass *eclass;
+    GEnumValue *ev;
+    gchar *endptr;
+    gint ret = 0;
+
+    ret = strtoul(string, &endptr, 0);
+    if (endptr != string) /* parsed a number */
+	return ret;
+
+    eclass = g_type_class_ref(type);
+    ev = g_enum_get_value_by_name(eclass, string);
+    if (!ev) ev = g_enum_get_value_by_nick(eclass, string);
+    if (ev)  ret = ev->value;
+
+    g_type_class_unref(eclass);
+
+    return ret;
+}
+
+/**
+ * glade_string_from_enum
+ * @type: the GType for this enum type.
+ * @value: the value of the enum
+ *
+ * This helper routine is designed to be used by widget build routines to
+ * convert the string representations of enumeration values found in the
+ * XML descriptions to the integer values that can be used to configure
+ * the widget.
+ *
+ * Returns: the string value for this enumeration
+ */
+const char *
+glade_string_from_enum (GType type, gint value)
+{
+    GEnumClass *eclass;
+    GEnumValue *ev;
+
+    eclass = g_type_class_ref(type);
+
+    ev = g_enum_get_value (eclass, value);
+
+    g_type_class_unref(eclass);
+
+    return ev ? ev->value_name : "0";
+}
+
+
+/**
+ * glade_util_flags_from_string
+ * @type: the GType for this flags type.
+ * @string: the string representation of the flags value.
+ *
+ * This helper routine is designed to be used by widget build routines
+ * to convert the string representations of flags values found in the
+ * XML descriptions to the integer values that can be used to
+ * configure the widget.  The string is composed of string names or
+ * nicknames for various flags separated by '|'.
+ *
+ * Returns: the integer value for this flags string
+ */
+guint
+glade_util_flags_from_string (GType type, const char *string)
+{
+    GFlagsClass *fclass;
+    gchar *endptr;
+    guint i, j, ret = 0;
+    char *flagstr;
+
+    ret = strtoul(string, &endptr, 0);
+    if (endptr != string) /* parsed a number */
+	return ret;
+
+    fclass = g_type_class_ref(type);
+
+
+    flagstr = g_strdup (string);
+    for (ret = i = j = 0; ; i++) {
+	gboolean eos;
+
+	eos = flagstr [i] == '\0';
+	
+	if (eos || flagstr [i] == '|') {
+	    GFlagsValue *fv;
+	    const char  *flag;
+
+	    flag = &flagstr [j];
+
+	    if (!eos) {
+		flagstr [i++] = '\0';
+		j = i;
+	    }
+
+	    fv = g_flags_get_value_by_name (fclass, flag);
+
+	    if (!fv)
+		fv = g_flags_get_value_by_nick (fclass, flag);
+
+	    if (fv)
+		ret |= fv->value;
+
+	    if (eos)
+		break;
+	}
+    }
+    
+    g_free (flagstr);
+
+    g_type_class_unref(fclass);
+
+    return ret;
+}
+
+/**
+ * glade_util_string_from_flags
+ * @type: the GType for this flags type.
+ * @flags: the value of the flags
+ *
+ * This helper routine is designed to be used by widget build routines
+ * to convert the flags values into the string representations in the
+ * XML descriptions.
+ * The string is composed of string names or
+ * nicknames for various flags separated by '|'.
+ *
+ * Returns: the string representation of these flags. Must be freed.
+ */
+gchar *
+glade_util_string_from_flags (GType type, guint flags)
+{
+    GFlagsClass *flags_class;
+    GString *string;
+    char *ret;
+
+    flags_class = g_type_class_ref (type);
+
+    string = g_string_new ("");
+
+    if (flags_class->n_values)
+      {
+	GFlagsValue *fval;
+      
+	for (fval = flags_class->values; fval->value_name; fval++)
+	  {
+	    /* We have to be careful as some flags include 0 values, e.g.
+	       BonoboDockItemBehavior uses 0 for BONOBO_DOCK_ITEM_BEH_NORMAL.
+	       If a 0 value is available, we only output it if the entire
+	       flags value is 0, otherwise we check if the bit-flag is set. */
+	    if ((fval->value == 0 && flags == 0)
+		|| (fval->value && (fval->value & flags) == fval->value))
+	      {
+		if (string->len)
+		  g_string_append_c (string, '|');
+		g_string_append (string, fval->value_name);
+	      }
+	  }
+      }
+
+    ret = string->str;
+    g_string_free (string, FALSE);
+
+    g_type_class_unref (flags_class);
+
+    return ret;
+}
+
+
+/* Returns TRUE if the widget should have a border width property.
+   Containers normally do, but Bonobo controls don't. */
+gboolean
+glade_util_uses_border_width (GtkWidget *widget)
+{
+#ifdef USE_GNOME
+  if (BONOBO_IS_WIDGET (widget))
+    return FALSE;
+#endif
+
+  if (GTK_IS_CONTAINER (widget))
+    return TRUE;
+
+  return FALSE;
+}
